use bson::doc;
use bson::Document;
use wccg_models::strava::{StravaAthlete, StravaAthleteSummary};
use wread_data_mongodb::crud_repository;
use wread_data_mongodb::mongodb::{error::Error, Database};

use wread_data_mongodb::mongodb::{
    options::UpdateModifications,
    results::{DeleteResult, InsertOneResult, UpdateResult},
};

const COLLECTION_NAME: &str = "stravariders";

pub async fn get_by_strava_athlete_id(
    strava_athlete_id: &String,
    db: &Database,
) -> Result<Option<StravaAthlete>, Error> {
    crud_repository::find_one(
        doc! {"stravaAthleteId": strava_athlete_id},
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn get_summary_by_strava_athlete_id(
    strava_athlete_id: &String,
    db: &Database,
) -> Result<Option<StravaAthleteSummary>, Error> {
    crud_repository::find_one(
        doc! {"stravaAthleteId": strava_athlete_id},
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn get_by_rider_id(
    rider_id: &String,
    db: &Database,
) -> Result<Option<StravaAthlete>, Error> {
    crud_repository::find_one(doc! {"riderId": rider_id}, COLLECTION_NAME, &db).await
}

pub async fn get_summary_by_rider_id(
    rider_id: &String,
    db: &Database,
) -> Result<Option<StravaAthleteSummary>, Error> {
    crud_repository::find_one(doc! {"riderId": rider_id}, COLLECTION_NAME, &db).await
}

pub async fn add(strava_athlete: &StravaAthlete, db: &Database) -> Result<InsertOneResult, Error> {
    crud_repository::add(strava_athlete, COLLECTION_NAME, db).await
}

pub async fn update(
    query: Document,
    update: impl Into<UpdateModifications>,
    db: &Database,
) -> Result<UpdateResult, Error> {
    crud_repository::update_one(query, update, None, COLLECTION_NAME, db).await
}

pub async fn delete_by_rider_id(rider_id: &String, db: &Database) -> Result<DeleteResult, Error> {
    let delete_query = doc! {"riderId": rider_id};
    crud_repository::delete_one(delete_query, None, COLLECTION_NAME, db).await
}
